@@ -25,7 +25,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
-#include "core/main-private.h"
 #include "meta/common.h"
 #include "meta/types.h"
 #include "ui/theme-private.h"
@@ -82,6 +81,8 @@ struct _MetaUIFrame
   MetaFrameControl prelit_control;
   MetaButtonState button_state;
   int grab_button;
+
+  gboolean is_frozen;
 };
 
 struct _MetaFrames
@@ -133,8 +134,9 @@ void meta_ui_frame_get_borders (MetaUIFrame      *frame,
 
 cairo_region_t * meta_ui_frame_get_bounds (MetaUIFrame *frame);
 
-void meta_ui_frame_get_mask (MetaUIFrame *frame,
-                             cairo_t     *cr);
+void meta_ui_frame_get_mask (MetaUIFrame           *frame,
+                             cairo_rectangle_int_t *frame_rect,
+                             cairo_t               *cr);
 
 void meta_ui_frame_move_resize (MetaUIFrame *frame,
                                 int x, int y, int width, int height);
